<?php

/**
 * This file contains \QUI\RabbitMQServer\Console
 */

namespace QUI\RabbitMQServer\Console;

use PCSG\Tasks\TestTask\TestTaskData;
use Psr\Log\LogLevel;
use QUI;
use QUI\Exception;
use QUI\RabbitMQServer\Interface\TaskSchedulerInterface;
use QUI\RabbitMQServer\Provider;
use QUI\System\License;

/**
 * Different console options for RabbitMQ module
 *
 * @author Patrick Müller (www.pcsg.de)
 */
class ConnectionTest extends QUI\System\Console\Tool
{
    /**
     * @param TaskSchedulerInterface|null $taskScheduler
     *
     * @throws Exception
     */
    public function __construct(
        private ?TaskSchedulerInterface $taskScheduler = null
    ) {
        if (is_null($this->taskScheduler)) {
            $provider = new Provider();
            $this->taskScheduler = $provider->getTaskScheduler();
        }

        $this->setName('quiqqer:rabbitmqserver')
            ->setDescription('Operations for RabbitMQ QUIQQER module');

        $this->addArgument(
            'action',
            'Action to execute: connectiontest, tasktest'
        );
    }

    /**
     * (non-PHPdoc)
     *
     * @see \QUI\System\Console\Tool::execute()
     */
    public function execute(): void
    {
        switch ($this->getArgument('action')) {
            case 'connectiontest':
                $this->connectionTest();
                break;

            case 'tasktest':
                $this->taskTest();
                break;

            default:
                $this->writeLn('Unknown action. Please only use: connectiontest');
        }

        $this->writeLn("\nFinished.\n");
    }

    private function connectionTest(): void
    {
        $this->writeLn(
            "Testing connection to RabbitMQ server with current settings..."
        );

        try {
            $server = (new Provider())->getQueueServer();
            $server->getChannel();
        } catch (\Exception $Exception) {
            $this->write(" [error] :: " . $Exception->getMessage());
            return;
        }

        $this->write(" [ok]");
    }

    /**
     * Test task system.
     *
     * @return void
     */
    private function taskTest(): void
    {
        $this->writeLn(
            "Testing connection with test task to RabbitMQ task system with current settings..."
        );

        $testTaskData = new TestTaskData(License::getSystemId());
        $queueJobUuid = $this->taskScheduler->queueWorkerTask($testTaskData, LogLevel::INFO);

        $this->writeLn("Test task {$queueJobUuid->uuid} successfully scheduled!");
        $this->writeLn("Please check INFO log files in this QUIQQER system and in the task system.");
        $this->writeLn("\nThe logs should include the QUIQQER system ID: " . License::getSystemId());
        $this->write(
            "\nWhere to check for logs and possible errors:"
            . "\n - QUIQQER log files: rabbitmq_tasks.info-" . date('Y-m-d') . ".log"
            . "\n - Task system log files: Check the log dir configured in the config.json of all connected task systems"
            . " (default: var/log/)"
            . "\n - Syslog of the task system service (systemctl status [task-service-name].service)"
        );
    }
}
